@@ -14,6 +14,7 @@ from . import cm
 from .axisgrid import Grid
 from .utils import (despine, axis_ticklabels_overlap, relative_luminance,
                     to_utf8)
+from ._decorators import _deprecate_positional_args
 
 
 __all__ = ["heatmap", "clustermap"]
@@ -65,7 +66,7 @@ def _matrix_mask(data, mask):
 
     """
     if mask is None:
-        mask = np.zeros(data.shape, np.bool)
+        mask = np.zeros(data.shape, bool)
 
     if isinstance(mask, np.ndarray):
         # For array masks, ensure that shape matches data then convert
@@ -75,7 +76,7 @@ def _matrix_mask(data, mask):
         mask = pd.DataFrame(mask,
                             index=data.index,
                             columns=data.columns,
-                            dtype=np.bool)
+                            dtype=bool)
 
     elif isinstance(mask, pd.DataFrame):
         # For DataFrame masks, ensure that semantic labels match data
@@ -195,7 +196,7 @@ class _HeatMapper(object):
         """Use some heuristics to set good defaults for colorbar and range."""
 
         # plot_data is a np.ma.array instance
-        calc_data = plot_data.filled(np.nan)
+        calc_data = plot_data.astype(float).filled(np.nan)
         if vmin is None:
             if robust:
                 vmin = np.nanpercentile(calc_data, 2)
@@ -297,9 +298,14 @@ class _HeatMapper(object):
         # Remove all the Axes spines
         despine(ax=ax, left=True, bottom=True)
 
+        # setting vmin/vmax in addition to norm is deprecated
+        # so avoid setting if norm is set
+        if "norm" not in kws:
+            kws.setdefault("vmin", self.vmin)
+            kws.setdefault("vmax", self.vmax)
+
         # Draw the heatmap
-        mesh = ax.pcolormesh(self.plot_data, vmin=self.vmin, vmax=self.vmax,
-                             cmap=self.cmap, **kws)
+        mesh = ax.pcolormesh(self.plot_data, cmap=self.cmap, **kws)
 
         # Set the axis limits
         ax.set(xlim=(0, self.data.shape[1]), ylim=(0, self.data.shape[0]))
@@ -347,12 +353,17 @@ class _HeatMapper(object):
             self._annotate_heatmap(ax, mesh)
 
 
-def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
-            annot=None, fmt=".2g", annot_kws=None,
-            linewidths=0, linecolor="white",
-            cbar=True, cbar_kws=None, cbar_ax=None,
-            square=False, xticklabels="auto", yticklabels="auto",
-            mask=None, ax=None, **kwargs):
+@_deprecate_positional_args
+def heatmap(
+    data, *,
+    vmin=None, vmax=None, cmap=None, center=None, robust=False,
+    annot=None, fmt=".2g", annot_kws=None,
+    linewidths=0, linecolor="white",
+    cbar=True, cbar_kws=None, cbar_ax=None,
+    square=False, xticklabels="auto", yticklabels="auto",
+    mask=None, ax=None,
+    **kwargs
+):
     """Plot rectangular data as a color-encoded matrix.
 
     This is an Axes-level function and will draw the heatmap into the
@@ -383,22 +394,23 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         If True, write the data value in each cell. If an array-like with the
         same shape as ``data``, then use this to annotate the heatmap instead
         of the data. Note that DataFrames will match on position, not index.
-    fmt : string, optional
+    fmt : str, optional
         String formatting code to use when adding annotations.
     annot_kws : dict of key, value mappings, optional
-        Keyword arguments for ``ax.text`` when ``annot`` is True.
+        Keyword arguments for :meth:`matplotlib.axes.Axes.text` when ``annot``
+        is True.
     linewidths : float, optional
         Width of the lines that will divide each cell.
     linecolor : color, optional
         Color of the lines that will divide each cell.
-    cbar : boolean, optional
+    cbar : bool, optional
         Whether to draw a colorbar.
     cbar_kws : dict of key, value mappings, optional
-        Keyword arguments for `fig.colorbar`.
+        Keyword arguments for :meth:`matplotlib.figure.Figure.colorbar`.
     cbar_ax : matplotlib Axes, optional
         Axes in which to draw the colorbar, otherwise take space from the
         main Axes.
-    square : boolean, optional
+    square : bool, optional
         If True, set the Axes aspect to "equal" so each cell will be
         square-shaped.
     xticklabels, yticklabels : "auto", bool, list-like, or int, optional
@@ -406,7 +418,7 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         the column names. If list-like, plot these alternate labels as the
         xticklabels. If an integer, use the column names but plot only every
         n label. If "auto", try to densely plot non-overlapping labels.
-    mask : boolean array or DataFrame, optional
+    mask : bool array or DataFrame, optional
         If passed, data will not be shown in cells where ``mask`` is True.
         Cells with missing values are automatically masked.
     ax : matplotlib Axes, optional
@@ -414,14 +426,14 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         Axes.
     kwargs : other keyword arguments
         All other keyword arguments are passed to
-        :func:`matplotlib.axes.Axes.pcolormesh`.
+        :meth:`matplotlib.axes.Axes.pcolormesh`.
 
     Returns
     -------
     ax : matplotlib Axes
         Axes object with the heatmap.
 
-    See also
+    See Also
     --------
     clustermap : Plot a matrix using hierachical clustering to arrange the
                  rows and columns.
@@ -435,7 +447,7 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         :context: close-figs
 
         >>> import numpy as np; np.random.seed(0)
-        >>> import seaborn as sns; sns.set()
+        >>> import seaborn as sns; sns.set_theme()
         >>> uniform_data = np.random.rand(10, 12)
         >>> ax = sns.heatmap(uniform_data)
 
@@ -489,7 +501,7 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
     .. plot::
         :context: close-figs
 
-        >>> ax = sns.heatmap(flights, center=flights.loc["January", 1955])
+        >>> ax = sns.heatmap(flights, center=flights.loc["Jan", 1955])
 
     Plot every other column label and don't plot row labels:
 
@@ -528,8 +540,6 @@ def heatmap(data, vmin=None, vmax=None, cmap=None, center=None, robust=False,
         >>> with sns.axes_style("white"):
         ...     f, ax = plt.subplots(figsize=(7, 5))
         ...     ax = sns.heatmap(corr, mask=mask, vmax=.3, square=True)
-
-
     """
     # Initialize the plotter object
     plotter = _HeatMapper(data, vmin, vmax, cmap, center, robust, annot, fmt,
@@ -728,8 +738,12 @@ class _DendrogramPlotter(object):
         return self
 
 
-def dendrogram(data, linkage=None, axis=1, label=True, metric='euclidean',
-               method='average', rotate=False, tree_kws=None, ax=None):
+@_deprecate_positional_args
+def dendrogram(
+    data, *,
+    linkage=None, axis=1, label=True, metric='euclidean',
+    method='average', rotate=False, tree_kws=None, ax=None
+):
     """Draw a tree diagram of relationships within a matrix
 
     Parameters
@@ -1220,31 +1234,34 @@ class ClusterGrid(Grid):
         return self
 
 
-def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
-               z_score=None, standard_scale=None, figsize=(10, 10),
-               cbar_kws=None, row_cluster=True, col_cluster=True,
-               row_linkage=None, col_linkage=None,
-               row_colors=None, col_colors=None, mask=None,
-               dendrogram_ratio=.2, colors_ratio=0.03,
-               cbar_pos=(.02, .8, .05, .18), tree_kws=None,
-               **kwargs):
+@_deprecate_positional_args
+def clustermap(
+    data, *,
+    pivot_kws=None, method='average', metric='euclidean',
+    z_score=None, standard_scale=None, figsize=(10, 10),
+    cbar_kws=None, row_cluster=True, col_cluster=True,
+    row_linkage=None, col_linkage=None,
+    row_colors=None, col_colors=None, mask=None,
+    dendrogram_ratio=.2, colors_ratio=0.03,
+    cbar_pos=(.02, .8, .05, .18), tree_kws=None,
+    **kwargs
+):
     """Plot a matrix dataset as a hierarchically-clustered heatmap.
 
     Parameters
     ----------
-    data: 2D array-like
+    data : 2D array-like
         Rectangular data for clustering. Cannot contain NAs.
     pivot_kws : dict, optional
         If `data` is a tidy dataframe, can provide keyword arguments for
         pivot to create a rectangular dataframe.
     method : str, optional
-        Linkage method to use for calculating clusters.
-        See scipy.cluster.hierarchy.linkage documentation for more information:
-        https://docs.scipy.org/doc/scipy/reference/generated/scipy.cluster.hierarchy.linkage.html
+        Linkage method to use for calculating clusters. See
+        :func:`scipy.cluster.hierarchy.linkage` documentation for more
+        information.
     metric : str, optional
         Distance metric to use for the data. See
-        scipy.spatial.distance.pdist documentation for more options
-        https://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.distance.pdist.html
+        :func:`scipy.spatial.distance.pdist` documentation for more options.
         To use different metrics (or methods) for rows and columns, you may
         construct each linkage matrix yourself and provide them as
         {row,col}_linkage.
@@ -1258,10 +1275,10 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
         Either 0 (rows) or 1 (columns). Whether or not to standardize that
         dimension, meaning for each row or column, subtract the minimum and
         divide each by its maximum.
-    figsize: (width, height), optional
+    figsize : (width, height), optional
         Overall size of the figure.
     cbar_kws : dict, optional
-        Keyword arguments to pass to ``cbar_kws`` in ``heatmap``, e.g. to
+        Keyword arguments to pass to ``cbar_kws`` in :func:`heatmap`, e.g. to
         add a label to the colorbar.
     {row,col}_cluster : bool, optional
         If True, cluster the {rows, columns}.
@@ -1276,11 +1293,11 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
         from the DataFrames column names or from the name of the Series.
         DataFrame/Series colors are also matched to the data by their
         index, ensuring colors are drawn in the correct order.
-    mask : boolean array or DataFrame, optional
+    mask : bool array or DataFrame, optional
         If passed, data will not be shown in cells where ``mask`` is True.
         Cells with missing values are automatically masked. Only used for
         visualizing, not for calculating.
-    {dendrogram,colors}_ratio: float, or pair of floats, optional
+    {dendrogram,colors}_ratio : float, or pair of floats, optional
         Proportion of the figure size devoted to the two marginal elements. If
         a pair is given, they correspond to (row, col) ratios.
     cbar_pos : (left, bottom, width, height), optional
@@ -1290,12 +1307,12 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
         Parameters for the :class:`matplotlib.collections.LineCollection`
         that is used to plot the lines of the dendrogram tree.
     kwargs : other keyword arguments
-        All other keyword arguments are passed to :func:`heatmap`
+        All other keyword arguments are passed to :func:`heatmap`.
 
     Returns
     -------
-    clustergrid : ClusterGrid
-        A ClusterGrid instance.
+    clustergrid : :class:`ClusterGrid`
+        A :class:`ClusterGrid` instance.
 
     Notes
     -----
@@ -1316,7 +1333,7 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
     .. plot::
         :context: close-figs
 
-        >>> import seaborn as sns; sns.set(color_codes=True)
+        >>> import seaborn as sns; sns.set_theme(color_codes=True)
         >>> iris = sns.load_dataset("iris")
         >>> species = iris.pop("species")
         >>> g = sns.clustermap(iris)
@@ -1375,8 +1392,6 @@ def clustermap(data, pivot_kws=None, method='average', metric='euclidean',
         :context: close-figs
 
         >>> g = sns.clustermap(iris, z_score=0, cmap="vlag")
-
-
     """
     plotter = ClusterGrid(data, pivot_kws=pivot_kws, figsize=figsize,
                           row_colors=row_colors, col_colors=col_colors,
